
!ChangedMessageSet methodsFor: '*packagesvscs-message list' stamp: 'nk 9/17/2003 16:27'!
addModelItemsToWindowMenu: aMenu
	"Add model-related item to the window menu"

	super addModelItemsToWindowMenu: aMenu. 
	aMenu addLine;
		add: 'remove all from change set' target: self action: #removeAllFromChangeSet;
		add: 'invert' target: self action: #diffFromChangeSet! !

!ChangedMessageSet methodsFor: '*packagesvscs-message list' stamp: 'nk 6/24/2003 14:20'!
canResetFilters
	"I can re-create my list, so answer true"
	^true! !

!ChangedMessageSet methodsFor: '*packagesvscs-message list' stamp: 'nk 6/24/2003 12:56'!
filterable
	"Answer whether the receiver can be changed by manual deletions"

	^ true! !

!ChangedMessageSet methodsFor: '*packagesvscs-message list' stamp: 'nk 6/24/2003 12:56'!
growable
	"Answer whether the receiver can be changed by manual additions"

	^ false! !

!ChangedMessageSet methodsFor: '*packagesvscs' stamp: 'nk 9/17/2003 16:33'!
removeAllFromChangeSet
	messageList do: [ :mr | changeSet removeSelectorChanges:  mr methodSymbol class: mr actualClass ]
! !

!ChangeSetCategory methodsFor: '*packagesvscs-services' stamp: 'nk 9/4/2004 15:55'!
fileOutAllChangeSets
	"File out all the nonempty change sets in the current category, 
	suppressing the checks for slips that might otherwise ensue. Obtain 
	user confirmation before undertaking this possibly prodigious task."
	| aList oldDir newDir |
	aList := self elementsInOrder
				select: [:aChangeSet | aChangeSet isEmpty not].
	aList size == 0
		ifTrue: [^ self inform: 'sorry, all the change sets in this category are empty'].
	(self confirm: 'This will result in filing out ' , aList size printString , ' change set(s) into the directory that you will be asked to select next.
Are you certain you want to do this?')
		ifFalse: [^ self].
	oldDir := ChangeSet defaultChangeSetDirectory.
	newDir := FileList2 modalFolderSelector: ChangeSet defaultChangeSetDirectory.
	newDir
		ifNil: [^ self].
	[ChangeSet defaultChangeSetDirectory: newDir.
	Preferences
		setFlag: #checkForSlips
		toValue: false
		during: [ChangeSorter fileOutChangeSetsNamed: (aList
					collect: [:m | m name]) asSortedArray]]
		ensure: [ChangeSet defaultChangeSetDirectory: oldDir]! !

!ChangeSetCategory methodsFor: '*packagesvscs-queries' stamp: 'nk 5/1/2004 10:08'!
isPredefined
	"Answer whether this is a category that was predefined and so can't be removed."

	^ true! !

!ChangeSet methodsFor: '*packagesvscs-packages' stamp: 'nk 8/26/2003 15:07'!
browseAfterFixingAllMethodsToBeInPackageNamed: aPackageName
	"Fix all of my methods so that they're included in aPackageName. Then browse the changed methods."
	| bad |
	bad _ self fixAllMethodsToBeInPackageNamed: aPackageName.
	bad isEmpty ifTrue: [ ^self inform: 'None!!' ].
	self systemNavigation 
		browseMessageList: bad 
		name: self name, ' methods that have been put in package ', aPackageName	! !

!ChangeSet methodsFor: '*packagesvscs-packages' stamp: 'nk 4/21/2004 17:52'!
browseAfterRemovingAllPackagePrefixes
	"Fix all of my methods so that they're not included in any packaege.
	Then browse the changed methods."
	| bad |
	bad _ self fixAllMethodsToBeInNoPackage.
	bad isEmpty ifTrue: [ ^self inform: 'None!!' ].
	self systemNavigation 
		browseMessageList: bad 
		name: self name, ' methods that have been removed from packages'! !

!ChangeSet methodsFor: '*packagesvscs-packages' stamp: 'nk 1/23/2004 07:54'!
browseAllExtensionMethodRemovals
	"Browse all the extension methods that I removed."
	| removals pkg |
	removals _ self extensionMethodRemovals.
	removals ifEmpty: [ ^self ].
	pkg _ (self probablePackage ifNotNilDo: [ :p | ' (package ', p packageName, ')' ]) ifNil: [ '' ].
	self systemNavigation 
		browseMessageList: self methodRemovals 
		name: self name, pkg, ' extension method removals'! !

!ChangeSet methodsFor: '*packagesvscs-packages' stamp: 'nk 6/13/2004 10:19'!
browseMessagesWithAnyOutOfOrderVersions
	"Open a message list browser on the new and changed methods in the receiver which have at least one prior version that is newer."

	| aList changeRecs stamps |

	aList _ self 
		messageListForChangesWhich: [ :aClass :aSelector |
			changeRecs _ aClass ifNotNilDo: [ :cls | cls changeRecordsAt: aSelector ].
			changeRecs notNil and: [ changeRecs size > 1 and: [
				stamps _ changeRecs collect: [ :ea | ea timeStamp ].
				(1 to: stamps size - 1) anySatisfy: [ :i |
					(stamps at: i) < (stamps at: i + 1) ]]]]
		ifNone: [^self "inform: 'none!!'"].
	self systemNavigation 
		browseMessageList: aList 
		name: self name, ' methods that have out of order versions'! !

!ChangeSet methodsFor: '*packagesvscs-packages' stamp: 'nk 6/13/2004 10:43'!
browseMessagesWithOutOfOrderVersions
	"Open a message list browser on the new and changed methods in 
	the receiver which have at least one prior version that is newer."
	| aList changeRecs myRec |
	aList := self
				messageListForChangesWhich: [:aClass :aSelector | 
					changeRecs := aClass
								ifNotNilDo: [:cls | cls changeRecordsAt: aSelector].
					changeRecs notNil
						and: [changeRecs size > 1
								and: [myRec := changeRecs
												findFirst: [:cr | self containsMethodAtPosition: cr position].
									myRec > 0
										and: [changeRecs size > myRec
												and: [(changeRecs at: myRec) timeStamp < (changeRecs at: myRec + 1) timeStamp]]]]]
				ifNone: [^ self"inform: 'none!!'"].
	self systemNavigation browseMessageList: aList name: self name , ' methods that have out of order versions'! !

!ChangeSet methodsFor: '*packagesvscs-packages' stamp: 'nk 8/26/2003 15:08'!
browseMethodsNotInCoreOfPackage: packageName 
	"Open a message list browser on the new and changed methods in the receiver which are not in the core classes of the PackageInfo named 'packageName'"

	| aList |
	aList := self methodsNotInCoreOfPackageNamed: packageName.
	aList isEmpty ifTrue: [^self inform: 'None!!'].
	self systemNavigation  browseMessageList: aList
		name: self name , ' methods that are not in core classes of package ' 
				, packageName! !

!ChangeSet methodsFor: '*packagesvscs-packages' stamp: 'nk 8/26/2003 15:08'!
browseMethodsNotInPackage: packageName 
	"Open a message list browser on the new and changed methods in the receiver which are not in the PackageInfo named 'packageName'"

	| aList |
	aList := self methodsNotInPackageNamed: packageName.
	aList isEmpty ifTrue: [^self inform: 'None!!'].
	self systemNavigation  browseMessageList: aList
		name: self name , ' methods that are not in package ' , packageName! !

!ChangeSet methodsFor: '*packagesvscs-packages' stamp: 'nk 6/14/2004 18:54'!
explodeByPackage
	"Make 0 or more new change sets with subsets of my changes that would belong to various projects."
	| cml allNew emptyCS |
	cml _ self changedMessageList.
	allNew _ Dictionary new.
	PackageOrganizer default packages do: [ :pkg | | pkgList newCSName newCS |
		pkgList _ cml select: [ :mr | pkg includesMethodReference: mr ].
		newCSName _ (self name, '-', pkg packageName).
		newCS _ ChangeSorter existingOrNewChangeSetNamed: newCSName.
		allNew at: pkg put: newCS.
		pkgList do: [ :mr | | sel |
			sel _ mr methodSymbol.
			(MessageSet isPseudoSelector: sel) ifFalse: [
			newCS absorbMethod: sel class: mr actualClass from: self ]].
		cml _ cml copyWithoutAll: pkgList.
	].
	allNew keysAndValuesDo: [ :pkg :cs |
		changeRecords keys do: [ :clsName | | realName parts |
			parts _ clsName asString findTokens: ' '.
			realName _ parts first asSymbol.
			Smalltalk at: realName ifPresentAndInMemory: [ :cls |
				(pkg includesClass: cls) ifTrue: [
					cs absorbClass: realName from: self.
"					self removeClassChanges: realName."
					cs absorbClass: (cls class) name from: self.
"					self removeClassChanges: (cls class) name "
					]]]].
"	self expungeEmptyClassChangeEntries."
	emptyCS := allNew select: [ :cs | cs isEmpty ].
	emptyCS do: [ :cs | ChangeSorter removeChangeSet: cs ].
	^allNew values copyWithoutAll: emptyCS.
! !

!ChangeSet methodsFor: '*packagesvscs-packages' stamp: 'nk 1/23/2004 08:24'!
extensionMethodRemovalRecords
	"Answer a collection of MethodReferences"
	^ Array
		streamContents: [:s | changeRecords
				keysAndValuesDo: [:className :classChangeRecord | 
					| realClass | 
					realClass := nil.
					classChangeRecord methodChanges
						keysAndValuesDo: [:selector :methodChangeRecord | 
							| info | 
							info := methodChangeRecord methodInfoFromRemoval.
							(info notNil
									and: [info second notNil
											and: [info second first = $*]])
								ifTrue: [realClass
										ifNil: [realClass := classChangeRecord realClass].
									realClass
										ifNotNil: [| splitInfo splitCat | 
											splitInfo := info second findTokens: '*-'.
											splitCat := realClass theNonMetaClass category findTokens: '-'.
											(splitInfo first sameAs: splitCat first)
												ifFalse: [s
														nextPut: (methodChangeRecord)]]]]]]! !

!ChangeSet methodsFor: '*packagesvscs-packages' stamp: 'nk 1/23/2004 08:06'!
extensionMethodRemovals
	"Answer a collection of MethodReferences"
	^ Array
		streamContents: [:s | changeRecords
				keysAndValuesDo: [:className :classChangeRecord | 
					| realClass | 
					realClass := nil.
					classChangeRecord methodChanges
						keysAndValuesDo: [:selector :methodChangeRecord | 
							| info | 
							info := methodChangeRecord methodInfoFromRemoval.
							(info notNil
									and: [info second notNil
											and: [info second first = $*]])
								ifTrue: [realClass
										ifNil: [realClass := classChangeRecord realClass].
									realClass
										ifNotNil: [| splitInfo splitCat | 
											splitInfo := info second findTokens: '*-'.
											splitCat := realClass theNonMetaClass category findTokens: '-'.
											(splitInfo first sameAs: splitCat first)
												ifFalse: [s
														nextPut: (MethodReference class: realClass selector: selector)]]]]]]! !

!ChangeSet methodsFor: '*packagesvscs-packages' stamp: 'nk 4/21/2004 18:16'!
fixAllMethodsToBeInNoPackage
	"For all of my method changes and additions, make sure that the  
	category prefixes do not have prefixes. 
	Answer the fixed methods."

	| bad cl org oldCat newCat index methodSymbol |
	bad := OrderedCollection new.

	self changedMessageListAugmented do: [ :each |
		cl _ each actualClass.
		methodSymbol _ each methodSymbol.
		(cl notNil and: [methodSymbol ~= 'Comment']) ifTrue: [
			org _ cl organization.
			oldCat _ (org categoryOfElement: methodSymbol) asString.
			oldCat first = $* ifTrue: [
				index _ oldCat indexOf: $- startingAt: 2 ifAbsent: [ oldCat size ].
				newCat _ oldCat copyFrom: index + 1 to: oldCat size.
				newCat isEmpty ifTrue: [ newCat _ 'previously in ', (oldCat copyFrom: 2 to: oldCat size) ].
				org classify: methodSymbol under: newCat suppressIfDefault: false.
				bad add: each.
			].
		]
	].

	^bad asArray.! !

!ChangeSet methodsFor: '*packagesvscs-packages' stamp: 'nk 6/20/2003 16:38'!
fixAllMethodsToBeInPackageNamed: aPackageName 
	"For all of my method changes and additions, make sure that the 
	category prefixes allow their inclusion in aPackageName. Answer the 
	fixed methods."
	| pi bad prefix redundant |
	pi _ PackageInfo named: aPackageName.

	bad _ self methodsNotInPackageNamed: aPackageName.
	redundant _ self methodsRedundantlyCategorizedForPackageNamed: aPackageName.
	bad isEmpty
		and: [redundant isEmpty
				ifTrue: [^ bad]].

	prefix _ pi methodCategoryPrefix allButFirst asLowercase.
	bad do: [:mr | PackageInfo reCategorizeForeignMethod: mr forPackageNamed: prefix ].

	redundant do: [:mr | PackageInfo reCategorizeOwnMethod: mr forPackageNamed: prefix ].
	^ bad , redundant! !

!ChangeSet methodsFor: '*packagesvscs-packages' stamp: 'nk 1/23/2004 07:55'!
methodRemovals
	"Answer a collection of MethodReferences"
	^ Array
		streamContents: [:s | changeRecords
				keysAndValuesDo: [:className :classChangeRecord | 
					| realClass | 
					realClass := nil.
					classChangeRecord methodChanges
						keysAndValuesDo: [:selector :methodChangeRecord | 
							| info | 
							info := methodChangeRecord methodInfoFromRemoval.
							info
								ifNotNil: [realClass
										ifNil: [realClass := classChangeRecord realClass].
									realClass
										ifNotNil: [s
												nextPut: (MethodReference class: realClass selector: selector)]]]]]! !

!ChangeSet methodsFor: '*packagesvscs-packages' stamp: 'nk 6/19/2003 19:31'!
methodsNotInCoreOfPackageNamed: aPackageName 
	"Answer a collection of MethodReferences for my method changes which aren't contained in aPackageName"

	| pi |
	pi := PackageInfo named: aPackageName.
	^self messageListForChangesWhich: 
			[:cl :ms | 
			cl notNil and: [ms ~= 'Comment' and: [(pi includesClass: cl) not]]]
		ifNone: [#()]! !

!ChangeSet methodsFor: '*packagesvscs-packages' stamp: 'nk 6/19/2003 19:30'!
methodsNotInPackageNamed: aPackageName 
	"Answer a collection of MethodReferences for my method changes which aren't contained in aPackageName"

	| pi |
	pi := PackageInfo named: aPackageName.
	^self messageListForChangesWhich: 
			[:cl :ms | 
			cl notNil 
				and: [ms ~= 'Comment' and: [(pi includesMethod: ms ofClass: cl) not]]]
		ifNone: [#()]! !

!ChangeSet methodsFor: '*packagesvscs-packages' stamp: 'nk 1/23/2004 07:16'!
methodsNotInProbablePackage
	"Answer a collection of MethodReferences for my method changes which aren't contained in my probable package"

	| pi |
	pi := self probablePackage.
	pi ifNil: [ ^#() ].
	^self methodsNotInPackageNamed: pi packageName.! !

!ChangeSet methodsFor: '*packagesvscs-packages' stamp: 'nk 6/19/2003 20:06'!
methodsRedundantlyCategorizedForPackageNamed: aPackageName 
	"Answer a collection of MethodReferences for my method changes which have a prefix for aPackageName but don't need one."

	| pi org cat |
	pi := PackageInfo named: aPackageName.
	^self messageListForChangesWhich: 
			[:cl :ms | 
			cl notNil and: 
					[ms ~= 'Comment' and: 
							[(pi includesClass: cl) and: 
									[org := cl organization.
									cat _ org categoryOfElement: ms.
									cat notNil and: [ pi isYourClassExtension: cat ]]]]]
		ifNone: [#()]! !

!ChangeSet methodsFor: '*packagesvscs-packages' stamp: 'nk 6/24/2004 18:23'!
probablePackage
	| ccl cml possible matches matching |
	ccl _ self changedClasses collect: [ :ea | ea category ] thenSelect: [ :ea | ea notNil ].
	possible _ PackageInfo allPackages select: [ :pkg |
		ccl anySatisfy: [ :c | pkg includesSystemCategory: c ]].
	possible isEmpty ifTrue: [ ^nil ].
	cml _ self changedMessageListAugmented.
	matches := IdentityDictionary new.
	possible do: [ :pkg |
		matching _ (cml count: [ :mr | pkg includesMethodReference: mr ]) / cml size.
		matching > 0.20 ifTrue: [ matches at: pkg put: matching ]].
	^matches keys detectMax:  [ :k | matches at: k ]
! !

!StaticChangeSetCategory methodsFor: '*packagesvscs-queries' stamp: 'nk 5/1/2004 10:10'!
isPredefined
	"Answer whether this is a category that was predefined and so can't be removed."

	^ false! !

!MessageSet methodsFor: '*packagesvscs-message list' stamp: 'nk 6/24/2003 12:55'!
addExtraShiftedItemsTo: aMenu 
	"The shifted selector-list menu is being built. Add items specific to 
	MessageSet "
	self filterable ifTrue: [
		aMenu addLine.
		aMenu add: 'remove from this browser' action: #removeMessageFromBrowser.
		aMenu add: 'filter message list...'  action: #filterMessageList.
		self canResetFilters ifTrue: [
			aMenu add: 'reset filters' action: #resetFilters]].
	self growable
		ifTrue: [aMenu add: 'add to message list...' action: #augmentMessageList].
	aMenu add: 'sort by date' action: #sortByDate! !

!MessageSet methodsFor: '*packagesvscs-filtering' stamp: 'nk 6/24/2003 14:21'!
canResetFilters
	"Answer true if I can reconstitute my list"
	^false! !

!MessageSet methodsFor: '*packagesvscs-filtering' stamp: 'nk 6/24/2003 12:46'!
filterMessageList
	"Allow the user to refine the list of messages."

	| aMenu evt |
	Smalltalk isMorphic ifFalse: [^ self inform: 'sorry, morphic only at this time.'].
	messageList size <= 1 ifTrue: [^ self inform: 'this is not a propitious filtering situation'].

	"would like to get the evt coming in but thwarted by the setInvokingView: circumlocution"
	evt _ self currentWorld activeHand lastEvent.
	aMenu _ MenuMorph new defaultTarget: self.
	aMenu addTitle: 'Filter by only showing...'.
	aMenu addStayUpItem.

	aMenu addList: #(
		('unsent messages'						filterToUnsentMessages		'filter to show only messages that have no senders')
		-
		('messages that send...'					filterToSendersOf			'filter to show only messages that send a selector I specify')
		('messages that do not send...'			filterToNotSendersOf		'filter to show only messages that do not send a selector I specify')
		-
		('messages whose selector is...'			filterToImplementorsOf		'filter to show only messages with a given selector I specify')
		('messages whose selector is NOT...'		filterToNotImplementorsOf	'filter to show only messages whose selector is NOT a selector I specify')
		-
		('messages in current change set'		filterToCurrentChangeSet	'filter to show only messages that are in the current change set')
		('messages not in current change set'	filterToNotCurrentChangeSet	'filter to show only messages that are not in the current change set')
		-
		('messages in any change set'			filterToAnyChangeSet		'filter to show only messages that occur in at least one change set')
		('messages not in any change set'		filterToNotAnyChangeSet		'filter to show only messages that do not occur in any change set in the system')
		-
		('messages in package...'				filterToMethodsInPackage	'filter to show only messages in a package that I specify')
		('messages not in package...'			filterToNotMethodsInPackage	'filter to show only messages NOT in a package that I specify')
		('messages in any package'				filterToMethodsInAnyPackage	'filter to show only messages in some package')
		('messages not in any package'			filterToMethodsNotInAnyPackage	'filter to show only messages in NO package')
		-
		('messages authored by me'				filterToCurrentAuthor		'filter to show only messages whose authoring stamp has my initials')
		('messages not authored by me'			filterToNotCurrentAuthor	'filter to show only messages whose authoring stamp does not have my initials')
		-
		('messages logged in .changes file'		filterToMessagesInChangesFile	'filter to show only messages whose latest source code is logged in the .changes file')
		('messages only in .sources file'			filterToMessagesInSourcesFile	'filter to show only messages whose latest source code is logged in the .sources file')
		-
		('messages with prior versions'			filterToMessagesWithPriorVersions	'filter to show only messages that have at least one prior version')
		('messages without prior versions'		filterToMessagesWithoutPriorVersions	'filter to show only messages that have no prior versions')
		-
		('uncommented messages' filterToUncommentedMethods 'filter to show only messages that do not have comments at the beginning')
		('commented messages' filterToCommentedMethods 'fileter to show only messages that have comments at the beginning')
		-
		('messages in hardened classes'			filterToMessagesWithHardenedClasses	'filter to show only messages of established classes (as opposed to Uniclasses such as Player23)')
		-
		('messages that...'						filterToMessagesThat			'let me type in a block taking a class and a selector, which will specify yea or nay concerning which elements should remain in the list')
			).

	aMenu popUpEvent: evt hand lastEvent in: evt hand world.! !

!MessageSet methodsFor: '*packagesvscs-filtering' stamp: 'nk 6/24/2003 14:21'!
filterToMethodsInAnyPackage
	"Display only methods that are represented by a PackageInfo"
	| packages |
	packages _ PackageInfo allPackages.
	self setFilteredList: (self messageList select: [ :mr |
			packages anySatisfy: [ :pi | pi includesMethodReference: mr ]
		])! !

!MessageSet methodsFor: '*packagesvscs-filtering' stamp: 'nk 6/24/2003 14:21'!
filterToMethodsInPackage
	"Display only methods that are represented by a specific PackageInfo"
	| packageNames choice pi |
	packageNames _ PackageInfo allPackageNames asSortedCollection.
	choice _ PopUpMenu withCaption: 'Which package?' chooseFrom: packageNames.
	choice isZero ifTrue: [ ^self ].
	choice _ packageNames at: choice.
	pi := PackageInfo named: choice.
	self setFilteredList: (self messageList select: [ :mr |
		pi includesMethodReference: mr ]
		)! !

!MessageSet methodsFor: '*packagesvscs-filtering' stamp: 'nk 6/24/2003 14:22'!
filterToMethodsNotInAnyPackage
	"Display only methods that are represented by no PackageInfo"
	| packages |
	packages _ PackageInfo allPackages.
	self setFilteredList: (self messageList select: [ :mr |
			packages noneSatisfy: [ :pi | pi includesMethodReference: mr ]
		])! !

!MessageSet methodsFor: '*packagesvscs-filtering' stamp: 'nk 6/24/2003 14:22'!
filterToNotMethodsInPackage
	"Display only methods that are not represented by a specific PackageInfo"
	| packageNames choice pi |
	packageNames _ PackageInfo allPackageNames asSortedCollection.
	choice _ PopUpMenu withCaption: 'Which package?' chooseFrom: packageNames.
	choice isZero ifTrue: [ ^self ].
	choice _ packageNames at: choice.
	pi := PackageInfo named: choice.
self setFilteredList: (self messageList select: [ :mr |
		(pi includesMethodReference: mr) not ]
		)! !

!MessageSet methodsFor: '*packagesvscs-message list' stamp: 'nk 6/24/2003 12:49'!
filterable
	"Answer whether the receiver is capable of growing/shrinking dynamically"

	^ true! !

!MessageSet methodsFor: '*packagesvscs-filtering' stamp: 'nk 6/24/2003 14:22'!
resetFilters
	"reconstitute my list without filters"
	self reformulateList! !

!RecentMessageSet methodsFor: '*packagesvscs-update' stamp: 'nk 6/24/2003 12:50'!
filterable
	"Answer whether the receiver can be changed by manual additions & deletions"

	^ false! !

!ChangeRecord methodsFor: '*packagesvscs-accessing' stamp: 'nk 6/24/2004 15:50'!
isExtensionMethod
	^ category beginsWith: '*'
! !

!ChangeRecord methodsFor: '*packagesvscs-accessing' stamp: 'nk 6/24/2004 16:03'!
package
	type == #method ifTrue: [ ^PackageOrganizer default packageOfMethodCategory: self category ofClass: self methodClass ifNone: [ ]].
	^PackageOrganizer default packageOfClass: self methodClass ifNone: [].
! !

!ChangeRecord methodsFor: '*packagesvscs-accessing' stamp: 'nk 6/13/2004 10:37'!
position
	^position! !

!MessageNames methodsFor: '*packagesvscs-selector list' stamp: 'nk 6/24/2003 14:20'!
canResetFilters
	"I can recreate my list, so answer true"
	^true! !

!MessageNames methodsFor: '*packagesvscs-selector list' stamp: 'nk 6/24/2003 14:20'!
resetFilters
	"reconstitute my list"
	messageList _ nil.
	self messageList.! !

!PackageInfo class methodsFor: '*packagesVsCS-enumeration' stamp: 'nk 4/16/2004 18:17'!
allPackageNames
	^PackageOrganizer default packages collect: [ :ea | ea packageName ]! !

!PackageInfo class methodsFor: '*packagesVsCS-enumeration' stamp: 'nk 2/23/2005 17:09'!
reCategorizeForeignMethod: aMethodReference forPackageNamed: packageName
	"Answer a (possibly modified) category name for the given method reference"
	| org oldCat newCat oldPrefix tokens | 
	org _ aMethodReference actualClass organization.
	oldCat _ (org categoryOfElement: aMethodReference methodSymbol) ifNil: [ '' ].
	oldCat _ oldCat asString.
	({ 'as yet unclassified'. 'all' } includes: oldCat) ifTrue: [ oldCat _ '' ].
	tokens _ oldCat findTokens: '*-' keep: '*'.

	"Strip off any old prefixes"
	((tokens at: 1 ifAbsent: [ '' ]) = '*') ifTrue: [
		[ ((tokens at: 1 ifAbsent: [ '' ]) = '*') ]
			whileTrue: [ tokens removeFirst ].
		oldPrefix _ tokens removeFirst asLowercase.
		[ (tokens at: 1 ifAbsent: [ '' ]) asLowercase = oldPrefix ]
			whileTrue: [ tokens removeFirst ].
	].

	newCat _ String streamContents: [ :s |
		s nextPut: $*; nextPutAll: packageName asLowercase.
		tokens do: [ :tok | s nextPut: $-; nextPutAll: tok ].
		(VersionsBrowser versionCountForSelector: aMethodReference methodSymbol class: aMethodReference actualClass) > 1 ifTrue: [ s nextPutAll: '-override' ].
	].

	org
		classify: aMethodReference methodSymbol
		under: newCat! !

!PackageInfo class methodsFor: '*packagesVsCS-enumeration' stamp: 'nk 6/20/2003 16:34'!
reCategorizeOwnMethod: aMethodReference forPackageNamed: packageName
	"Answer a (possibly modified) category name for the given method reference"
	| org oldCat newCat oldPrefix tokens | 
	org _ aMethodReference actualClass organization.
	oldCat _ (org categoryOfElement: aMethodReference methodSymbol) ifNil: [ '' ].
	oldCat _ oldCat asString.
	({ 'as yet unclassified'. 'all' } includes: oldCat) ifTrue: [ oldCat _ '' ].
	tokens _ oldCat findTokens: '*-' keep: '*'.

	"Strip off any old prefixes"
	((tokens at: 1 ifAbsent: [ '' ]) = '*') ifTrue: [
		[ ((tokens at: 1 ifAbsent: [ '' ]) = '*') ]
			whileTrue: [ tokens removeFirst ].
		oldPrefix _ tokens removeFirst asLowercase.
		[ (tokens at: 1 ifAbsent: [ '' ]) asLowercase = oldPrefix ]
			whileTrue: [ tokens removeFirst ].
	].

	newCat _ String streamContents: [ :s |
		tokens do: [ :tok | s nextPutAll: tok ]
			separatedBy: [ s nextPut: $- ].
	].

	org
		classify: aMethodReference methodSymbol
		under: newCat! !

!ChangeSetCategoryWithParameters methodsFor: '*packagesvscs-queries' stamp: 'nk 5/1/2004 10:08'!
acceptsManualAdditions
	"Answer whether the user is allowed manually to manipulate the contents of the change-set-category."

	^ false! !

!ChangeSetCategoryWithParameters methodsFor: '*packagesvscs-queries' stamp: 'nk 5/1/2004 10:10'!
isPredefined
	"Answer whether this is a category that was predefined and so can't be removed."

	^ true! !

!ChangeSorter class methodsFor: '*packagesvscs-class initialization' stamp: 'nk 6/24/2004 18:14'!
addMonticelloCategory
	"Create a new, static change-set category, which will be populated entirely by change sets whose names contain MC followed by digits."

	| catName aCategory |
	catName _ #'MC change sets'.
	aCategory _ (ChangeSetCategories includesKey: catName) ifFalse: [
		aCategory _ ChangeSetCategoryWithParameters new categoryName: catName.
		aCategory membershipSelector: #isMCChangeSet: ; parameters: {}.
		ChangeSetCategories elementAt: catName put: aCategory.
		aCategory reconstituteList.
	].
! !

!ChangeSorter class methodsFor: '*packagesvscs-services' stamp: 'nk 6/29/2004 21:15'!
changeSet: aChangeSet belongsToPackage: aPackageInfo 
	| ccl cml matching |
	ccl _ aChangeSet changedClasses collect: [ :ea | ea category ] thenSelect: [ :ea | ea notNil ].
	ccl isEmpty ifTrue: [ ^false ].
	ccl do: [ :c | (aPackageInfo includesSystemCategory: c ) ifTrue: [ ^true ]].
	cml _ aChangeSet changedMessageListAugmented.
	cml isEmpty ifTrue: [ ^false ].
	matching := (cml count: [ :mr | aPackageInfo includesMethodReference: mr ]) / cml size.
	^matching > 0.20! !

!ChangeSorter class methodsFor: '*packagesvscs-class initialization' stamp: 'nk 2/12/2004 10:18'!
initialize
	"Initialize the class variables"

	AllChangeSets ifNil: [AllChangeSets _ OrderedCollection new].
	self gatherChangeSets.
	ChangeSetCategories ifNil: [self initializeChangeSetCategories].
	RecentUpdateMarker _ 0.

	"ChangeSorter initialize"

	FileList registerFileReader: self.

	self registerInFlapsRegistry.
! !

!ChangeSorter class methodsFor: '*packagesvscs-class initialization' stamp: 'nk 2/12/2004 10:13'!
initializeChangeSetCategories
	"Initialize the set of change-set categories"

	"ChangeSorter initializeChangeSetCategories"

	| aCategory |
	ChangeSetCategories := ElementCategory new 
				categoryName: #ChangeSetCategories.

	aCategory := ChangeSetCategory new categoryName: #All.
	aCategory membershipSelector: #belongsInAll:.
	aCategory documentation: 'All change sets known to the system'.
	ChangeSetCategories addCategoryItem: aCategory.

	aCategory := ChangeSetCategory new categoryName: #Additions.
	aCategory membershipSelector: #belongsInAdditions:.
	aCategory 
		documentation: 'All unnumbered change sets except those representing projects in the system as initially released.'.
	ChangeSetCategories addCategoryItem: aCategory.

	aCategory := ChangeSetCategory new categoryName: #MyInitials.
	aCategory membershipSelector: #belongsInMyInitials:.
	aCategory 
		documentation: 'All change sets whose names end with the current author''s initials.'.
	ChangeSetCategories addCategoryItem: aCategory.

	aCategory := ChangeSetCategory new categoryName: #Numbered.
	aCategory membershipSelector: #belongsInNumbered:.
	aCategory 
		documentation: 'All change sets whose names start with a digit -- normally these will be the official updates to the system.'.
	ChangeSetCategories addCategoryItem: aCategory.

	aCategory := ChangeSetCategory new categoryName: #ProjectChangeSets.
	aCategory membershipSelector: #belongsInProjectChangeSets:.
	aCategory 
		documentation: 'All change sets that are currently associated with projects present in the system right now.'.
	ChangeSetCategories addCategoryItem: aCategory.
	
	aCategory := ChangeSetCategory new categoryName: #ProjectsInRelease.
	aCategory membershipSelector: #belongsInProjectsInRelease:.
	aCategory 
		documentation: 'All change sets belonging to projects that were shipped in the initial release of this version of Squeak'.
	ChangeSetCategories addCategoryItem: aCategory.

	aCategory := ChangeSetCategory new categoryName: #RecentUpdates.
	aCategory membershipSelector: #belongsInRecentUpdates:.
	aCategory 
		documentation: 'Updates whose numbers are at or beyond the number I have designated as the earliest one to qualify as Recent'.
	ChangeSetCategories addCategoryItem: aCategory.

	self addMonticelloCategory.

	ChangeSetCategories elementsInOrder 
		do: [:anElem | anElem reconstituteList]! !

!ChangeSorter class methodsFor: '*packagesvscs-enumerating' stamp: 'nk 5/1/2004 09:46'!
isMCChangeSet: aChangeSet
	| index |
	index _ aChangeSet name findString: 'MC' startingAt: 1.
	index = 0 ifTrue: [ ^false ].
	^(aChangeSet name at: index+2 ifAbsent: [ $- ]) isDigit! !

!ChangeSorter class methodsFor: '*packagesvscs-removing' stamp: 'nk 2/23/2005 07:20'!
removeEmptyUnnamedChangeSets
	"Remove all change sets that are empty, whose names start with 
	Unnamed, and which are not nailed down by belonging to a 
	Project. Also remove those change sets created by Monticello that 
	are empty."
	"ChangeSorter removeEmptyUnnamedChangeSets"
	| toGo message |
	toGo := (self
				changeSetsNamedSuchThat: [:csName | (csName beginsWith: 'Unnamed')
						or: [(csName beginsWith: 'MC')
								and: [(csName copyFrom: 3 to: csName size)
										allSatisfy: [:c | c isDigit]]]])
				select: [:cs | cs isEmpty
						and: [cs hasPreamble not
								and: [cs hasPostscript not
										and: [cs okayToRemoveInforming: false]]]].
	message := String
				streamContents: [:s | 
					s print: toGo size;
						 nextPutAll: ' change set(s) removed.';
						 cr.
					toGo
						do: [:cs | s nextPutAll: cs name]
						separatedBy: [s nextPutAll: ', ']].
	toGo
		do: [:cs | self removeChangeSet: cs].
	self inform: message! !

!ChangeSorter class methodsFor: '*packagesvscs-services' stamp: 'nk 5/1/2004 09:16'!
reorderChangeSets
	"Change the order of the change sets to something more convenient:
		First come the project changesets that come with the release.  These are mostly empty.
		Next come all numbered updates.
		Next come all remaining changesets
	In a ChangeSorter, they will appear in the reversed order."

	"ChangeSorter reorderChangeSets"

	| newHead newMid newTail |
	newHead _ SortedCollection sortBlock: [ :a :b | b name < a name ].
	newMid _ SortedCollection sortBlock: [ :a :b | b name < a name ].
	newTail _ SortedCollection sortBlock: [ :a :b | b name < a name ].
	AllChangeSets do:
		[:aChangeSet |
			(self belongsInProjectsInRelease: aChangeSet)
				ifTrue:
					[newHead add: aChangeSet]
				ifFalse:
					[(self belongsInNumbered: aChangeSet)
						ifTrue:
							[newMid add: aChangeSet]
						ifFalse:
							[newTail add: aChangeSet]]].
	AllChangeSets _ OrderedCollection new.
	AllChangeSets addAllLast: newHead.
	AllChangeSets addAllLast: newMid.
	AllChangeSets addAllLast: newTail.
	Smalltalk isMorphic ifTrue: [SystemWindow wakeUpTopWindowUponStartup]! !

!ChangeSorter methodsFor: '*packagesvscs-drag and drop' stamp: 'nk 9/4/2004 15:18'!
acceptDroppingMorph: transferMorph event: evt inMorph: dstListMorph 
	"Here we are fetching information from the dropped transferMorph 
	and performing the correct action for this drop."
	| selector |
	selector := self selectorForHandlingDropOf: transferMorph onMorph: dstListMorph.
	selector ifNil: [ ^false ].

	Symbol hasInterned: selector ifTrue: [ :sym |
		^(self respondsTo: sym)
			ifTrue: [ self perform: sym with: transferMorph with: dstListMorph. ]
			ifFalse: [ false ] ].
	! !

!ChangeSorter methodsFor: '*packagesvscs-changeSet menu' stamp: 'nk 9/4/2004 15:34'!
addAllToCategoryOpposite
	"Add the current change set to the category viewed on the opposite side, if it's of the sort to accept things like that"

	| categoryOpposite otherSide |
	categoryOpposite _ (otherSide := parent other: self) changeSetCategory.
	categoryOpposite acceptsManualAdditions
		ifTrue:
			[ self changeSetCategory elementsInOrder do: [ :cs | categoryOpposite addChangeSet: cs ].
			categoryOpposite reconstituteList.
			otherSide update]
		ifFalse:
			[self inform: 
'sorry, this command only makes sense
if the category showing on the opposite
side is a static category whose
members are manually maintained']! !

!ChangeSorter methodsFor: '*packagesvscs-changeSet menu' stamp: 'nk 9/4/2004 15:34'!
addToCategoryOpposite
	"Add the current change set to the category viewed on the opposite side, if it's of the sort to accept things like that"

	| categoryOpposite otherSide |
	categoryOpposite _ (otherSide := parent other: self) changeSetCategory.
	categoryOpposite acceptsManualAdditions
		ifTrue:
			[categoryOpposite addChangeSet: myChangeSet.
			categoryOpposite reconstituteList.
			otherSide update]
		ifFalse:
			[self inform: 
'sorry, this command only makes sense
if the category showing on the opposite
side is a static category whose
members are manually maintained']! !

!ChangeSorter methodsFor: '*packagesvscs-changeSet menu' stamp: 'nk 6/24/2003 14:19'!
addToPackage
	"Prompt for a package name, then fix all of the methods in this change set so they are in the package. Display a message list with the changed messages"
	| packageNames choice |
	packageNames _ PackageInfo allPackageNames asSortedCollection.
	choice _ PopUpMenu withCaption: 'Add methods to which package?' chooseFrom: packageNames.
	choice isZero ifTrue: [ ^self ].
	myChangeSet browseAfterFixingAllMethodsToBeInPackageNamed: (packageNames at: choice)
! !

!ChangeSorter methodsFor: '*packagesvscs-changeSet menu' stamp: 'nk 6/13/2004 10:00'!
browseMessagesWithOutOfOrderVersions
	myChangeSet browseMessagesWithOutOfOrderVersions! !

!ChangeSorter methodsFor: '*packagesvscs-changeSet menu' stamp: 'nk 9/4/2004 15:38'!
categorySubmenu: aMenu  shifted: shiftedIgnored
	"Fill aMenu with less-frequently-needed category items"
	
	aMenu title: 'Change set category'.
	aMenu addStayUpItem.

	aMenu addList: #(
		('make a new category...' makeNewCategory 'Creates a new change-set-category (you will be asked to supply a name) which will start out its life with this change set in it')
		('make a new category with class...' makeNewCategoryShowingClassChanges 'Creates a new change-set-category that includes change sets that change a particular class (you will be asked to supply a name)')
		('rename this category' renameCategory 'Rename this change-set category.   Only applies when the current category being shown is a manually-maintained, user-defined category, such as you can create for yourself by choosing "make a new category..." from this same menu.')
		('remove this category' removeCategory 'Remove this change-set category.   Only applies when the current category being shown is a manually-maintained, user-defined category, such as you can create for yourself by choosing "make a new category..." from this same menu.')
		('show categories of this changeset' showCategoriesOfChangeSet 'Show a list of all the change-set categories that contain this change-set; if the you choose one of the categories from this pop-up, that category will be installed in this change sorter')
	-).

	parent ifNotNil:
		[aMenu addList: #(
			('add change set to category opposite' addToCategoryOpposite 'Adds this change set to the category on the other side of the change sorter.  Only applies if the category shown on the opposite side is a manually-maintained, user-defined category, such as you can create for yourself by choosing "make a new category..." from this same menu.')
			('add all change sets to category opposite' addAllToCategoryOpposite 'Adds all of the change sets in the category shown on this side to the category on the other side of the change sorter.  Only applies if the category shown on the opposite side is a manually-maintained, user-defined category, such as you can create for yourself by choosing "make a new category..." from this same menu.')
			('remove all change sets from category opposite' removeAllFromCategoryOpposite 'Removes all of the change sets in the category shown on this side from the category on the other side of the change sorter.  Only applies if the category shown on the opposite side is a manually-maintained, user-defined category, such as you can create for yourself by choosing "make a new category..." from this same menu.')

		)].

	aMenu addList: #(
		('remove change set from this category' removeFromCategory 'Removes this change set from the current category.  Only applies when the current category being shown is a manually-maintained, user-defined category, such as you can create for yourself by choosing "make a new category..." from this same menu.')
		-
		('file out category''s change sets' fileOutAllChangeSets 'File out every change set in this category that has anything in it.  The usual checks for slips are suppressed when this command is done.')
		('set recent-updates marker' setRecentUpdatesMarker 'Allows you to specify a number that will demarcate which updates are considered "recent" and which are not.  This will govern which updates are included in the RecentUpdates category in a change sorter')
		('fill aggregate change set' fillAggregateChangeSet 'Creates a change-set named Aggregate into which all the changes in all the change sets in this category will be copied.')
		-
		('back to main menu' offerUnshiftedChangeSetMenu 'Takes you back to the shifted change-set menu.')
		('back to shifted menu' offerShiftedChangeSetMenu 'Takes you back to the primary change-set menu.')).

	^ aMenu! !

!ChangeSorter methodsFor: '*packagesvscs-drag and drop' stamp: 'nk 9/4/2004 15:11'!
dragPassengerFor: item inMorph: dragSource 
	| transferType |
	(dragSource isKindOf: PluggableListMorph)
		ifFalse: [^nil].
	transferType _ self dragTransferTypeForMorph: dragSource.
	^transferType caseOf: {
		[#classList] -> [ self selectedClassOrMetaClass   ].
		[#changeSetList] -> [ self changeSet ].
		[#messageList] -> [ self selectedClassOrMetaClass  -> self selectedMessageName ].
	} otherwise: [].
! !

!ChangeSorter methodsFor: '*packagesvscs-drag and drop' stamp: 'nk 9/4/2004 14:39'!
dragTransferTypeForMorph: dragSource 
	^(dragSource isKindOf: PluggableListMorph)
		ifTrue: [dragSource getListSelector]! !

!ChangeSorter methodsFor: '*packagesvscs-changeSet menu' stamp: 'nk 6/14/2004 18:54'!
explodeByPackage
	| newCS |
	newCS := myChangeSet explodeByPackage.
	newCS do: [ :cs | 
		myChangeSet forgetAllChangesFoundIn: cs.
		self changeSetCategory acceptsManualAdditions
			ifTrue: [ self changeSetCategory addChangeSet: cs ]].
	myChangeSet expungeEmptyClassChangeEntries.
	self update.! !

!ChangeSorter methodsFor: '*packagesvscs-drag and drop' stamp: 'nk 9/4/2004 15:23'!
handleDropFromChangeSetList: aTransferMorph toChangeSetList: aListMorph
	"add change set to category"
	self changeSetCategory acceptsManualAdditions
		ifFalse: [ self inform: 'nope!!' ]
		ifTrue: [ self changeSetCategory addChangeSet: aTransferMorph passenger; reconstituteList.
			self update. ]! !

!ChangeSorter methodsFor: '*packagesvscs-drag and drop' stamp: 'nk 9/4/2004 15:18'!
handleDropFromClassList: aTransferMorph toChangeSetList: aListMorph
	"add class to change set"
self halt! !

!ChangeSorter methodsFor: '*packagesvscs-drag and drop' stamp: 'nk 9/4/2004 15:18'!
handleDropFromMessageList: aTransferMorph toChangeSetList: aListMorph
	"add message to change set"
self halt! !

!ChangeSorter methodsFor: '*packagesvscs-changeSet menu' stamp: 'nk 6/29/2004 21:09'!
makeNewCategoryForSelectedPackage
	| names pkgName |
	names := PackageOrganizer default packageNames asSortedCollection.
	pkgName := PopUpMenu withCaption: 'For which package?' chooseFrom: names.
	pkgName > 0 ifTrue: [
			self makeNewCategoryForPackage: (PackageOrganizer default packageNamed: (names at: pkgName) ifAbsent: [^self ]) ].! !

!ChangeSorter methodsFor: '*packagesvscs-drag and drop' stamp: 'nk 9/4/2004 18:28'!
openAsMorphIn: window rect: rect
	"Add a set of change sorter views to the given top view offset by the given amount. To create a single change sorter, call this once with an offset of 0@0. To create a dual change sorter, call it twice with offsets of 0@0 and 0.5@0."

	| csListHeight msgListHeight csMsgListHeight |
	contents _ ''.
	csListHeight _ 0.25.
	msgListHeight _ 0.25.
	csMsgListHeight _ csListHeight + msgListHeight.
	self addDependent: window.		"so it will get changed: #relabel"
	
"The method SystemWindow>>addMorph:fullFrame: checks scrollBarsOnRight, then adds the morph at the back if true, otherwise it is added in front. But flopout hScrollbars needs the crrentSelector pane to be behind the upper ones in the draw order. Hence the value of scrollBarsOnRight affects the order in which the lowerpanes are added."
	Preferences scrollBarsOnRight ifFalse:
		[window addMorph: (PluggableListMorphByItem on: self
					list: #messageList
					selected: #currentSelector
					changeSelected: #currentSelector:
					menu: #messageMenu:shifted:
					keystroke: #messageListKey:from:)
			frame: (((0@csListHeight extent: 1@msgListHeight)
				scaleBy: rect extent) translateBy: rect origin)].

	window addMorph: ((PluggableListMorphByItem on: self
				list: #changeSetList
				selected: #currentCngSet
				changeSelected: #showChangeSetNamed:
				menu: #changeSetMenu:shifted:
				keystroke: #changeSetListKey:from:)
			autoDeselect: false)
		frame: (((0@0 extent: 0.5@csListHeight)
			scaleBy: rect extent) translateBy: rect origin).

	window addMorph: (PluggableListMorphByItem on: self
				list: #classList
				selected: #currentClassName
				changeSelected: #currentClassName:
				menu: #classListMenu:shifted:
				keystroke: #classListKey:from:)
		frame: (((0.5@0 extent: 0.5@csListHeight)
			scaleBy: rect extent) translateBy: rect origin).

	Preferences scrollBarsOnRight ifTrue:
		[window addMorph: (PluggableListMorphByItem on: self
					list: #messageList
					selected: #currentSelector
					changeSelected: #currentSelector:
					menu: #messageMenu:shifted:
					keystroke: #messageListKey:from:)
			frame: (((0@csListHeight extent: 1@msgListHeight)
				scaleBy: rect extent) translateBy: rect origin)].

	window submorphsDo: [ :m | m enableDragNDrop: true ].

	 self addLowerPanesTo: window
		at: (((0@csMsgListHeight corner: 1@1) scaleBy: rect extent) translateBy: rect origin)
		with: nil.
! !

!ChangeSorter methodsFor: '*packagesvscs-changeSet menu' stamp: 'nk 9/4/2004 15:36'!
removeAllFromCategoryOpposite
	"Remove all of my category's change sets from the category viewed on the opposite side, if it's of the sort to accept things like that"

	| categoryOpposite otherSide |
	categoryOpposite _ (otherSide := parent other: self) changeSetCategory.
	categoryOpposite acceptsManualAdditions
		ifTrue:
			[ self changeSetCategory elementsInOrder do: [ :cs | categoryOpposite removeElementAt: cs name ].
			categoryOpposite reconstituteList.
			otherSide update]
		ifFalse:
			[self inform: 
'sorry, this command only makes sense
if the category showing on the opposite
side is a static category whose
members are manually maintained']! !

!ChangeSorter methodsFor: '*packagesvscs-changeSet menu' stamp: 'nk 5/1/2004 10:11'!
removeCategory
	"Remove the current category"

	| itsName |
	self changeSetCategory isPredefined ifTrue:
		[^ self inform: 'sorry, you can only remove manually-added categories.'].

	(self confirm: 'Really remove the change-set-category
named ', (itsName _ changeSetCategory categoryName), '?') ifFalse: [^ self].

	ChangeSetCategories removeElementAt: itsName.
	self setDefaultChangeSetCategory.

	self update! !

!ChangeSorter methodsFor: '*packagesvscs-changeSet menu' stamp: 'nk 4/17/2004 17:44'!
removeEmptyUnnamedChangeSets
	self class removeEmptyUnnamedChangeSets.
	self update.
	self changed: #changeSetList.
	self changed: #currentCngSet! !

!ChangeSorter methodsFor: '*packagesvscs-changeSet menu' stamp: 'nk 4/21/2004 17:51'!
removeFromPackages
	myChangeSet browseAfterRemovingAllPackagePrefixes
! !

!ChangeSorter methodsFor: '*packagesvscs-changeSet menu' stamp: 'nk 5/1/2004 10:10'!
renameCategory
	"Obtain a new name for the category and, if acceptable, apply it"

	| catName oldName |
	self changeSetCategory isPredefined ifTrue:
		[^ self inform: 'sorry, you can only rename manually-added categories.'].

	catName _ FillInTheBlank request: 'Please give the new category a name' initialAnswer:  (oldName _ changeSetCategory categoryName).
	catName isEmptyOrNil ifTrue: [^ self].
	(catName _ catName asSymbol) = oldName ifTrue: [^ self inform: 'no change.'].
	(ChangeSetCategories includesKey: catName) ifTrue:
		[^ self inform: 'Sorry, there is already a category of that name'].

	changeSetCategory categoryName: catName.
	ChangeSetCategories removeElementAt: oldName.
	ChangeSetCategories elementAt: catName put: changeSetCategory.

	self update! !

!ChangeSorter methodsFor: '*packagesvscs-changeSet menu' stamp: 'nk 4/17/2004 17:44'!
reorderChangeSets
	"apply a standard reordering -- let the class handle this"

	self class reorderChangeSets.
	self changed: #changeSetList.
	self changed: #currentCngSet! !

!ChangeSorter methodsFor: '*packagesvscs-drag and drop' stamp: 'nk 9/4/2004 14:57'!
selectorForHandlingDropOf: transferMorph onMorph: destinationLM 

	| selector |

	selector _ [ String streamContents: [ :s |
		s nextPutAll: 'handleDropFrom';
			nextPutAll: transferMorph dragTransferType asString capitalized;
			nextPutAll: ':to';
			nextPutAll: destinationLM getListSelector asString capitalized;
			nextPut: $: ]] on: MessageNotUnderstood do: [ :ex | ex return: nil ].

	^selector! !

!ChangeSorter methodsFor: '*packagesvscs-changeSet menu' stamp: 'nk 6/13/2004 10:00'!
shiftedChangeSetMenu: aMenu
	"Set up aMenu to hold items relating to the change-set-list pane when the shift key is down"

	Smalltalk isMorphic ifTrue:
		[aMenu title: 'Change set (shifted)'.
		aMenu addStayUpItemSpecial].

	"CONFLICTS SECTION"
	aMenu add: 'conflicts with other change sets' action: #browseMethodConflicts.
	aMenu balloonTextForLastItem: 
'Browse all methods that occur both in this change set and in at least one other change set.'.
	parent ifNotNil:
		[aMenu add: 'conflicts with change set opposite' action: #methodConflictsWithOtherSide.
			aMenu balloonTextForLastItem: 
'Browse all methods that occur both in this change set and in the one on the opposite side of the change sorter.'.

			aMenu add: 'conflicts with category opposite' action: #methodConflictsWithOppositeCategory.
			aMenu balloonTextForLastItem: 
'Browse all methods that occur both in this change set and in ANY change set in the category list on the opposite side of this change sorter, other of course than this change set itself.  (Caution -- this could be VERY slow)'].
	aMenu addLine.

	"CHECKS SECTION"
	aMenu add: 'check for slips' action: #lookForSlips.
	aMenu balloonTextForLastItem: 
'Check this change set for halts and references to Transcript.'.

	aMenu add: 'check for unsent messages' action: #checkForUnsentMessages.
	aMenu balloonTextForLastItem:
'Check this change set for messages that are not sent anywhere in the system'.

	aMenu add: 'check for uncommented methods' action: #checkForUncommentedMethods.
	aMenu balloonTextForLastItem:
'Check this change set for methods that do not have comments'.

	aMenu add: 'check for uncommented classes' action: #checkForUncommentedClasses.
	aMenu balloonTextForLastItem:
'Check for classes with code in this changeset which lack class comments'.

	Utilities authorInitialsPerSe isEmptyOrNil ifFalse:
		[aMenu add: 'check for other authors' action: #checkForAlienAuthorship.
		aMenu balloonTextForLastItem:
'Check this change set for methods whose current authoring stamp does not start with "', Utilities authorInitials, '"'.

	aMenu add: 'check for any other authors' action: #checkForAnyAlienAuthorship.
	aMenu balloonTextForLastItem:
'Check this change set for methods any of whose authoring stamps do not start with "', Utilities authorInitials, '"'].

	aMenu add: 'check for uncategorized methods' action: #checkForUnclassifiedMethods.
	aMenu balloonTextForLastItem:
'Check to see if any methods in the selected change set have not yet been assigned to a category.  If any are found, open a browser on them.'.

	aMenu add: 'check for out of order versions' action: #browseMessagesWithOutOfOrderVersions.
	aMenu balloonTextForLastItem:
'Check to see if any methods in the selected change set have out-of-order versions in the past.  If any are found, open a browser on them.'.

	aMenu addLine.

	aMenu add: 'inspect change set' action: #inspectChangeSet.
	aMenu balloonTextForLastItem: 
'Open an inspector on this change set. (There are some details in a change set which you don''t see in a change sorter.)'.

	aMenu add: 'remove empty unnamed change sets' action: #removeEmptyUnnamedChangeSets.
	aMenu balloonTextForLastItem: 
'Removes all the empty change sets with names like Unnamed* or MC<digit>+'.

	aMenu add: 'add all methods to package...' action: #addToPackage.
	aMenu balloonTextForLastItem:
'Prompts for a package to add all the change set''s methods to, then fixes their category names and opens a browser on the changed methods.'.

	aMenu add: 'remove all methods from packages' action: #removeFromPackages.
	aMenu balloonTextForLastItem:
'Removes package prefixes from all of my methods, and opens a browser on the changed methods.'.

	aMenu add: 'explode by package' action: #explodeByPackage.
	aMenu balloonTextForLastItem:
'Separates the methods in the selected change set into separate change sets by package ownership'.

	aMenu add: 'update' action: #update.
	aMenu balloonTextForLastItem: 
'Update the display for this change set.  (This is done automatically when you activate this window, so is seldom needed.)'.

	aMenu add: 'go to change set''s project' action: #goToChangeSetsProject.
	aMenu balloonTextForLastItem: 
'If this change set is currently associated with a Project, go to that project right now.'.

	aMenu add: 'promote to top of list' action: #promoteToTopChangeSet.
	aMenu balloonTextForLastItem:
'Make this change set appear first in change-set lists in all change sorters.'.

	aMenu add: 'trim history' action: #trimHistory.
	aMenu balloonTextForLastItem: 
' Drops any methods added and then removed, as well as renaming and reorganization of newly-added classes.  NOTE: can cause confusion if later filed in over an earlier version of these changes'.

	aMenu add: 'remove contained in class categories...'
		action: #removeContainedInClassCategories.
	aMenu 
		balloonTextForLastItem: ' Drops any changes in given class categories'.
		
	aMenu add: 'clear this change set' action: #clearChangeSet.
	aMenu balloonTextForLastItem: 
'Reset this change set to a pristine state where it holds no information. CAUTION: this is destructive and irreversible!!'.
	aMenu add: 'expunge uniclasses' action: #expungeUniclasses.
	aMenu balloonTextForLastItem:
'Remove from the change set all memory of uniclasses, e.g. classes added on behalf of etoys, fabrik, etc., whose classnames end with a digit.'.

	aMenu add: 'uninstall this change set' action: #uninstallChangeSet.
	aMenu balloonTextForLastItem: 
'Attempt to uninstall this change set. CAUTION: this may not work completely and is irreversible!!'.

	aMenu addLine.
	aMenu add: 'file into new...' action: #fileIntoNewChangeSet.
	aMenu balloonTextForLastItem: 
'Load a fileout from disk and place its changes into a new change set (seldom needed -- much better to do this from a file-list browser these days.)'.

	aMenu add: 'reorder all change sets' action: #reorderChangeSets.
	aMenu balloonTextForLastItem:
'Applies a standard reordering of all change-sets in the system -- at the bottom will come the sets that come with the release; next will come all the numbered updates; finally, at the top, will come all other change sets'.


	aMenu addLine.

	aMenu add: 'more...' action: #offerUnshiftedChangeSetMenu.
	aMenu balloonTextForLastItem: 
'Takes you back to the primary change-set menu.'.

	^ aMenu! !

!ChangeSorter methodsFor: '*packagesvscs-drag and drop' stamp: 'nk 9/4/2004 15:12'!
wantsDroppedMorph: transferMorph event: anEvent inMorph: destinationLM 
	"We are only interested in TransferMorphs as wrappers for             
	informations. If their content is really interesting for us, will             
	determined later in >>acceptDroppingMorph:event:."

	| selector wanted |

	wanted := [ transferMorph source model ~~ self ] on: MessageNotUnderstood do: [ :ex | ex return: false ].
	wanted ifFalse: [ ^false ].

	selector _ self selectorForHandlingDropOf: transferMorph onMorph: destinationLM.

	Symbol hasInterned: selector ifTrue: [ :sym | ^self respondsTo: sym ].

	^false! !
ChangeSorter initialize!
